function FocusMapper(pMapperID, pRow, pCol) {
	_this_ = this;
	var focusMap = [];
	var mapperID = pMapperID;
	var pageLength = 0;
	var rowLength = pRow;
	var colLength = pCol;
	var curPosition = { page : 0, row : 0, col : 0 };
	var nItemLength = 0;
	
	this.generate = function(pPages) {
		curPosition.page =  0;
		this.refresh(pPages);
		this.resetFocus();
	}
	
	this.refresh = function(pPages) {
		nItemLength = 0;
		pageLength = pPages.length;
		
		// Album 
		focusMap.length = pageLength;
		
		for (var pageIdx = 0; pageIdx < pageLength; pageIdx++) {
			// Initialize mapper
			clearMap(pageIdx);
			var list = pPages[pageIdx];
			var listLength = list.length;
			var itemLength = 0;
			for (var listIdx = 0, rowIdx = 0; listIdx < listLength; listIdx++) {
				if (list[listIdx] instanceof Array) {
					itemLength = list[listIdx].length;
					for (var itemIdx = 0; itemIdx < itemLength; itemIdx++) {
						if (list[listIdx][itemIdx].disable === true) {
							focusMap[pageIdx][rowIdx][itemIdx].focus = false;
						} else {
							focusMap[pageIdx][rowIdx][itemIdx].focus = true;
						}
						nItemLength++;
					}
					rowIdx++;
				}
			}
		}
	}
	
	this.debug = function() {
		FBTrace("[Focus Map]");
		var strRow = "";
		for (var rowIdx = 0; rowIdx < rowLength; rowIdx++)	 {
			for (var colIdx = 0; colIdx < colLength; colIdx++) {
				strRow += focusMap[curPosition.page][rowIdx][colIdx].focus == true ? "O" : "X";
				strRow += "  ";
			}
			FBTrace(strRow);
			strRow = "";
		}
	}
	
	this.isFocusable = function(pPosition) {
		return focusMap[curPosition.page][pPosition.row][pPosition.col].focus;
	}
	
	this.setPage = function(pPage) {
		curPosition.page = pPage;
		clearItems(pPage);
	}
	
	this.getFocus = function(pItem) {
		return curPosition;
	}
	
	this.setFocus = function(pPosition) {
		for (var name in pPosition) {
			curPosition[name] = pPosition[name];
		}
	}
	
	this.getTotalPage = function() {
		return pageLength;
	}
	
	// Return 1-based current page
	this.getCurrentPage = function() {
		return curPosition.page + 1;
	}
	
	this.getItemLength = function() {
		return nItemLength;
	}
	
	this.resetFocus = function() {
		FocusToFirstItem();
	}
	
	this.focus = function(pFocusPosition) {
		var items = focusMap[curPosition.page][pFocusPosition.row][pFocusPosition.col].items;
		var itemLength = items.length;
		for (var item = 0; item < itemLength; item++) {
			items[item].focus(_document.getElementById(items[item].id));
		}
	}

	this.blur = function(pFocusPosition) {
		var items = focusMap[curPosition.page][pFocusPosition.row][pFocusPosition.col].items;
		var itemLength = items.length;
		for (var item = 0; item < itemLength; item++) {
			items[item].blur(_document.getElementById(items[item].id));
		}
	}
	
	this.addDiv = function(pFocusPosition, pContents, pStyle, pFocusFn, pBlurFn) {
		var strHTML =  "";
		var tMapItem = focusMap[curPosition.page][pFocusPosition.row][pFocusPosition.col];
		var tID = "_"+mapperID+"_" + pFocusPosition.row + "_" + pFocusPosition.col + "_" + tMapItem.items.length;
		tMapItem.items.push(
			{
				id : tID,
				focus : pFocusFn,
				blur : pBlurFn
			}
		);
		strHTML += "<div id = '"+tID+"' style = '"+pStyle+"'>";
		strHTML += pContents ? pContents : "";
		strHTML += "</div>"
		return strHTML;
	}
	
	this.addImage = function(pFocusPosition, pSrc, pStyle, pFocusFn, pBlurFn) {
		var strHTML =  "";
		var tMapItem = focusMap[curPosition.page][pFocusPosition.row][pFocusPosition.col];
		var tID = "_"+mapperID+"_" + pFocusPosition.row + "_" + pFocusPosition.col + "_" + tMapItem.items.length;
		tMapItem.items.push(
			{
				id : tID,
				focus : pFocusFn,
				blur : pBlurFn
			}
		);
		strHTML += "<img id = '"+tID+"' style = '"+pStyle+"' src = '"+pSrc+"'>";
		return strHTML;
	}
	
	this.getIndex = function() {
		return curPosition.page * rowLength * colLength + curPosition.row * colLength + curPosition.col + 1;
	}
	
	this.next = function() {
		if (curPosition.col < (colLength - 1)) {
			if (focusMap[curPosition.page][curPosition.row][curPosition.col+1].focus) {
				curPosition.col++;				
			} else {
				curPosition.page = 0;
				curPosition.row = 0;
				curPosition.col = 0;
				FocusToFirstItem();
			}
		} else if (curPosition.row < (rowLength - 1)) {
			if (focusMap[curPosition.page][curPosition.row + 1][0].focus) {
				curPosition.row++;
				curPosition.col = 0;
			}
			else {
				curPosition.page = 0;
				curPosition.row = 0;
				curPosition.col = 0;
				FocusToFirstItem();
			}
		} else if (curPosition.page < (pageLength - 1)) {
			if (focusMap[curPosition.page + 1][0][0].focus) {
				curPosition.page++;
				curPosition.row = 0;
				curPosition.col = 0;
			}
			else {
				curPosition.page = 0;
				curPosition.row = 0;
				curPosition.col = 0;
				FocusToFirstItem();
			}
		} else {
//					FocusToFirstItem();
			curPosition.page = 0;
			curPosition.row = 0;
			curPosition.col = 0;
		}
		return true;
	}
	
	this.prev = function() {
		if (curPosition.col  > 0) {
			curPosition.col--;
		} else {
			if (curPosition.row > 0) {
				curPosition.row--;
				curPosition.col = colLength - 1;
			} else {
				if (curPosition.page > 0) {
					curPosition.page--;
					curPosition.row = rowLength - 1;
					curPosition.col = colLength - 1;
				} else {
					curPosition.page = pageLength - 1;
					FocusToLastItem();
				}
			}
		}
		return true;
	}

	this.left = function(pRotate) {
		var bAction = false;
		if (colLength == 1) return bAction;
		for (var colIdx = curPosition.col - 1; colIdx > -1; colIdx--) {
			if (focusMap[curPosition.page][curPosition.row][colIdx].focus) {
				curPosition.col = colIdx;
				bAction = true;
				return bAction;
			}
		}
		if (pRotate) {
			for (var colIdx = colLength - 1; colIdx > curPosition.col; colIdx--) {
				if (focusMap[curPosition.page][curPosition.row][colIdx].focus) {
					curPosition.col = colIdx;
					bAction = true;
					return bAction;
				}
			}
		}
		return bAction;
	}

	this.right = function(pRotate) {
		var bAction = false;
		for (var colIdx = curPosition.col + 1; colIdx < colLength; colIdx++) {
			if (focusMap[curPosition.page][curPosition.row][colIdx].focus) {
				curPosition.col = colIdx;
				bAction = true;
				return bAction;
			}
		}
		if (pRotate) {
			for (var colIdx = 0; colIdx < curPosition.col; colIdx++) {
				if (focusMap[curPosition.page][curPosition.row][colIdx].focus) {
					curPosition.col = colIdx;
					bAction = true;
					return bAction;
				}
			}
		}
		return bAction;
	}

	this.down = function() {
		var bAction = false;
		var bFound = false;
		if (curPosition.row < (rowLength-1) && focusMap[curPosition.page][curPosition.row + 1][curPosition.col].focus) {
			curPosition.row++;
			bAction = true;
			return bAction;
		} else if (curPosition.row == (rowLength-1)) {
			this.nextPage();
			bAction = false;
		}
		else {
			for (var rowIdx = curPosition.row + 1; rowIdx < rowLength; rowIdx++) {
				for (var colIdx = 0; colIdx < colLength; colIdx++) {
					if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
						curPosition.row = rowIdx;
						curPosition.col = colIdx;
						bAction = true;
						bFound = true;
						return bAction;
					}
				}
			}
			if (!bFound) {
				this.nextPage();
				bAction = false;
			}
		}
		return bAction;
	}
	
	this.up = function() {
		var bAction = false;
		var bFound = false;
		if (curPosition.row > 0 && focusMap[curPosition.page][curPosition.row - 1][curPosition.col].focus) {
			curPosition.row--;
			bAction = true;
			return bAction;
		} else if (curPosition.row == 0) {
			this.prevPage();
			bAction = false;
		}
		else {
			for (var rowIdx = curPosition.row - 1; rowIdx > -1; rowIdx--) {
				for (var colIdx = 0; colIdx < colLength; colIdx++) {
					if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
						curPosition.row = rowIdx;
						curPosition.col = colIdx;
						bAction = true;
						bFound = true;
						return bAction;
					}
				}
			}
			if (!bFound) {
				this.prevPage();
				bAction = false;
			}
		}
		return bAction;
	}
	
	this.prevPage = function(pFocusReset) {
		if (pageLength == 1) return false;
		if (curPosition.page > 0) {
			curPosition.page--;
		} else {
			curPosition.page = pageLength - 1;
		}
		if (pFocusReset) FocusToFirstItem();
		else {
			var nLastRow = findLastRow();
			curPosition.row = nLastRow;
			if (!focusMap[curPosition.page][curPosition.row][curPosition.col].focus) {
				for (var colIdx = 0;colIdx < colLength; colIdx++) {
					if (focusMap[curPosition.page][curPosition.row][colIdx].focus) {
						curPosition.col = colIdx;
						break;
					}
				}
			} 
		}
		return true;
	}

	this.nextPage = function(pFocusReset) {
		if (pageLength == 1) return false;
		if (curPosition.page < (pageLength - 1)) {
			curPosition.page++;
		} else {
			curPosition.page = 0;
		}
		if (pFocusReset) FocusToFirstItem();
		else {
			curPosition.row = 0;
			if (!focusMap[curPosition.page][curPosition.row][curPosition.col].focus) {
				for (var rowIdx = curPosition.row; rowIdx < rowLength; rowIdx++) {
					for (var colIdx = 0; colIdx < colLength; colIdx++) {
						if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
							curPosition.row = rowIdx;
							curPosition.col = colIdx;
							bAction = true;
							bFound = true;
							return bAction;
						}
					}
				}
			} 
		}
		return true;
	}

	function findLastRow() {
		var nLastRowIdx = 0;
		var nColLength = 0;
		var bFound = false;
		for (var rowIdx = rowLength - 1; rowIdx > -1; rowIdx--) {
			nColLength = focusMap[curPosition.page][rowIdx].length;
			for (var colIdx = 0; colIdx < nColLength; colIdx++) {
				if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
					nLastRowIdx = rowIdx;
					bFound = true;
					break;
				}
			}
			if (bFound) break;
		}
		return nLastRowIdx;
	}

	function clearMap(pPage) {
		focusMap[pPage] = [];
		for (var rowIdx = 0; rowIdx < rowLength; rowIdx++)	 {
			focusMap[pPage][rowIdx] = [];
			for (var colIdx = 0; colIdx < colLength; colIdx++) {
				focusMap[pPage][rowIdx][colIdx] = {
					focus : false,
					items : []
				};
			}
		}
		
	}
	
	function clearItems(pPage) {
		for (var rowIdx = 0; rowIdx < rowLength; rowIdx++)	 {
			for (var colIdx = 0; colIdx < colLength; colIdx++) {
				focusMap[pPage][rowIdx][colIdx].items = [];
			}
		}
	}
	
	function FocusToFirstItem() {
		for (var rowIdx = 0; rowIdx < rowLength; rowIdx++)	 {
			for (var colIdx = 0; colIdx < colLength; colIdx++) {
				if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
					curPosition.row = rowIdx;
					curPosition.col = colIdx;
					return;
				}
			}
		}
	}
	
	function FocusToLastItem() {
		for (var rowIdx = (rowLength-1); rowIdx > -1; rowIdx--)	 {
			for (var colIdx = (colLength-1); colIdx > -1; colIdx--) {
				if (focusMap[curPosition.page][rowIdx][colIdx].focus) {
					curPosition.row = rowIdx;
					curPosition.col = colIdx;
					return;
				}
			}
		}
	}
}
