var NETSocket = {}

// XHR Widget Timeout
NETSocket.Timeout = {
	timer : null
	, timeStamp : null
	, nSec : 35
	, callbackFn : null
	, setTimer : function() {
		if (this.timer) clearTimeout(this.timer);
		this.timeStamp = (new Date()).getTime();
		this.timer = setTimeout(function(pTimeStamp){
			return function(){
				if (NETSocket.Timeout.callbackFn instanceof Function && NETSocket.Timeout.timeStamp ==  pTimeStamp) {
					alert("[NETSocket.Timeout] Widget XMLHttpRequest Timeout Occured. ");
					NETSocket.Timeout.callbackFn(false);
				}
			}(this.timeStamp);
		}, this.nSec * 1000);
	}
	, clearTimer : function() {
		if (this.timer) clearTimeout(this.timer);		
	}
	, setCallback : function() {
		
	}
}

/**
 * @brief           getXMLHttpRequest
 * @remarks      Get XMLHttpRequest
 * @param			userXHRObject
 * @return			XMLHttpRequest
*/
NETSocket.getXMLHttpRequest = function(userXHRObject){
//	TRACE("NETSocket.getXMLHttpRequest Start");
	var tmpXHRObject = userXHRObject;
	if (tmpXHRObject != null) {
		tmpXHRObject.destroy();
		tmpXHRObject = new XMLHttpRequest();
	} else {
		tmpXHRObject = new XMLHttpRequest();
	}
    
    if (!tmpXHRObject) {
        TRACE("Cannot create XMLHTTP Instance");
        return false;
    }
//	TRACE("NETSocket.getXMLHttpRequest End");
    return tmpXHRObject;
}

/**
 * @brief           setRequestHeader
 * @remarks      Set Request Header
 * @param			XmlHttpRequest Object
 * @param			Header name
 * @param			Contents
 * @return			Success | Fail
*/
NETSocket.setRequestHeader = function (XHRObject, header, body) {
    try {
        if (!XHRObject) {
            return false;
        } else {
            XHRObject.setRequestHeader(header, body);
            return true;
        }
    } catch (e) {
	    TRACE (e);
    }
}

/**
 * @brief           setURL
 * @remarks      Set URL to XHR
 * @param			XmlHttpRequest Object
 * @param			Method
 * @param			Request URL
 * @param			Parameter
 * @param			Callback Function
 * @param			userData
*/
NETSocket.setURL = function(XHRObject, method, url, param, callback){
    try {
		var fnUserCallback = function(){
			if (XHRObject.readyState == 3) {
			}
			else 
				if (XHRObject.readyState == 4) {
					NETSocket.callBack(XHRObject, XHRObject.status, callback);
				}
		}
        XHRObject.onreadystatechange = fnUserCallback;
//		NETSocket.Timeout.setCallback(fnUserCallback);
        if (param.length > 0) {
            if (param.charAt(0) == '?') {
                url = url + param;
            }
            else {
                url = url + "?" + param;
            }
        }
		TRACE("OpenAPISetURL(" + url + ")");
        XHRObject.open(method, url, true);
//        TRACE("OpenAPISetURL() End");
        return true;
    } 
    catch (e) {
        TRACE(e);
    }
}

/**
 * @brief           callURL
 * @remarks      Send Reqeust
 * @param			XmlHttpRequest Object
 * @param			Parameter
*/
NETSocket.callURL = function (XHRObject, param) {
	try {
		TRACE("NETSocket.callURL()");
        if (!XHRObject) {
            return false;
        }
        else {
//			NETSocket.Timeout.setTimer();
            if (param == "") {
                XHRObject.send(null);
            }
            else {
                XHRObject.send(param);
            }
        }
//		TRACE("NETSocket.callURL() End");
	} catch (e) {
		TRACE(e)
	}
}

/**
 * @brief           callBack
 * @remarks      Callback function for response
 * @param			XmlHttpRequest Object
 * @param			Stat
 * @param			Callback Function
 * @param			User Data
*/
NETSocket.callBack = function (XHRObject, stat, callbackFn){
	TRACE ("NETSocket.callBack()");
	try {
	    var result;
		if (stat == "200") {
			callbackFn(XHRObject);
		}
		else {
			alert(XHRObject.responseText);
			alert("stat : " + stat);
			callbackFn(false);
		}
	} catch (e) {
		TRACE (e);
	}
//	TRACE ("NETSocket.callBack() End");
}
