CSlideshow.Define = {
	// play state
	PLAY : 1,
	PAUSE : 2
}
CSlideshow.Image = function (filename, url, width, height, date, type, index, total) {
	this.filename = filename;
	this.url = url;
	this.width = width;
	this.height = height;
	this.date = date;
	this.type = type;
	this.index = index;
	this.total = total;
}
CSlideshow.Interval = function (name, time) {
	this.name = name;
	this.time = time;
}

function CSlideshow (strName) {
	strName = (strName === undefined) ? 'Slideshow' : strName;
	
	var eAnchor = null;
	var _THIS_ = this;
	var WIDGET = new Common.API.Widget();
	
	var iShow = new CSlideshow.Show('iShow');
	var fnOnReturn = null;
	
	this.Image = CSlideshow.Image;
	this.Interval = CSlideshow.Interval;
	this.open = function () {
		alert(strName + ' open()');
		iShow.open();
	}
	this.close = function () {
		alert(strName + ' close()');
		iShow.close();
	}
	this.focus = function (fn) {
		alert(strName + ' focus()');
		if (eAnchor == null) {
			// insert anchor
			var divId = "_div_for_CSlideshow_";
			var divNode = document.createElement("div");
			divNode.id = divId;
			document.body.appendChild(divNode);
			
			document.getElementById(divId).innerHTML = '<a href="javascript:void(0);" id="_anchor_for_CSlideshow_"></a>';
			eAnchor = document.getElementById('_anchor_for_CSlideshow_');
			eAnchor.addEventListener('onkeydown', function(){
				keyHandler();
			});
		}
		fnOnReturn = fn;
		eAnchor.focus();
	}
	this.getKeyHandler = function () {
		return function (keyCode) {
			keyHandler(keyCode);
		}
	}
	this.addSource = function (arrImage)  {
		alert(strName + ' addSource('+arrImage+')');
		if (!validation(arrImage)) {
			alert('Wrong parameter. arrImage must be the array of CSlideshow.Image() or CSlideshow().Image()');
			return;
		}
		iShow.addSource(arrImage);
		
		function validation (arrImage) {
			var retValue = true;
			var arrLength = arrImage.length;
			var constructor = null;
			for (var i=0; i < arrLength; i++) {
				constructor = arrImage[i].constructor;
				if (constructor != CSlideshow.Image && constructor != _THIS_.Image) {
					retValue = false;
				}
			}
			alert('Source validation returns ' + retValue);
			return retValue;
		}
	}
	this.addInterval = function (arrInterval)  {
		alert(strName + ' addInterval('+arrInterval+')');
		if (!validation(arrInterval)) {
			alert('Wrong parameter. arrInterval must be the array of CSlideshow.Interval() or CSlideshow().Interval()');
			return;
		}
		iShow.addInterval(arrInterval);
		
		function validation (arrInterval) {
			var retValue = true;
			var arrLength = arrInterval.length;
			var constructor = null;
			for (var i=0; i < arrLength; i++) {
				constructor = arrInterval[i].constructor;
				if (constructor != CSlideshow.Interval) {
					retValue = false;
				}
			}
			alert('Interval validation returns ' + retValue);
			return retValue;
		}
	}
	this.setNotifier = function (count, fn) {
		iShow.setNotifier(count, fn);
	}
	function keyHandler (keyCode) {
		keyCode = (keyCode === undefined) ? event.keyCode : keyCode;
		alert(strName + ' keyHandler('+keyCode+')');
		switch (keyCode) {
			case TVKEY.KEY_LEFT:
			case TVKEY.KEY_PANEL_VOL_DOWN:
				iShow.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_RIGHT:
			case TVKEY.KEY_PANEL_VOL_UP:
				iShow.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_UP:
			case TVKEY.KEY_WHEELUP:
			case TVKEY.KEY_PANEL_CH_UP:
				break;
				
			case TVKEY.KEY_DOWN:
			case TVKEY.KEY_WHEELDOWN:
			case TVKEY.KEY_PANEL_CH_DOWN:
				break;
				
			case TVKEY.KEY_ENTER:
			case TVKEY.KEY_PANEL_ENTER:
			case TVKEY.KEY_PANEL_SOURCE:
				iShow.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_RETURN:
			case TVKEY.KEY_PANEL_MENU:
				WIDGET.blockNavigation(event);
				if (fnOnReturn) {
					fnOnReturn();
					fnOnReturn = null;
				}
				break;
				
			case TVKEY.KEY_RW:
				iShow.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_FF:
				iShow.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_INFO:
				break;
				
			case TVKEY.KEY_PLAY:
				break;
				
			case TVKEY.KEY_PAUSE:
				break;
				
			case TVKEY.KEY_TOOLS:
				break;
	
			case TVKEY.KEY_BLUE:
				break;
				
			case TVKEY.KEY_MENU:
				location.reload();
	            break;
		}
	}
}

CSlideshow.Show = function (strName) {
	strName = (strName === undefined) ? 'Show' : strName;
	
	var bPlayMode = false;			// 일반 재생 / Slideshow 재생을 구분, true일 경우 buffering complete 이벤트를 무시함.
	var bEffectProcess = false;		// Transition effect 중에는 key input을 막음
	var bNextNotSupport = false;	// 다음 사진이 재생불가 포맷일 경우 effect를 none 으로 설정함.
	var bNetworkError = false;		// Network error가 났을 경우 다음 사진의 effect를 none으로 설정함.
	var bPrepared = false;			// decoding complete시 true, effect 수행이 가능함을 나타냄.
	var bRendered = false;			// rendering complete시 true, 다음 이미지의 preparing 시작이 가능함을 나타냄.
	
	var defPlayState = CSlideshow.Define.PAUSE;
	var defEffect = CImageViewer.Effect.FADE1;
	
	var MAX_SOURCE = 2000;
	var arrSource = null;
	var nSourceIdx = 0;
	var nNextIdx = 0;
	
	var fnNotifier = null;
	var nNotifyCount = 1;
	
	var _THIS_ = this;
	
	var iInterval = new Interval();
	var iImageViewer = new CImageViewer('ImageViewer');
	var iView = new View();
	
	iInterval.setOnEvent(function () {
		if (bPrepared && defPlayState == CSlideshow.Define.PLAY) {
			nextSlide();
		}
	});
	
	// for key handling
	this.keyHandler = function (keyCode) {
		keyCode = (keyCode === undefined) ? event.keyCode : keyCode;
		alert(strName + ' keyHandler('+keyCode+')');
		if (bEffectProcess) {
			alert('transition effect is operating. skip.');
			return;
		}
		switch (keyCode) {
			case TVKEY.KEY_LEFT:
			case TVKEY.KEY_PANEL_VOL_DOWN:
				if (arrSource.length == 1) return;
				if (defPlayState == CSlideshow.Define.PLAY) {
					iInterval.stop();
					iImageViewer.stop();
				}
				// adjust nSourceIdx
				if (--nSourceIdx < 0) {
					alert('this is the first photo. go to last.');
					nSourceIdx = arrSource.length - 1;
				}
				playImage(nSourceIdx);
				break;
				
			case TVKEY.KEY_RIGHT:
			case TVKEY.KEY_PANEL_VOL_UP:
				if (arrSource.length == 1) return;
				if (defPlayState == CSlideshow.Define.PLAY) {
					iInterval.stop();
					iImageViewer.stop();
				}
				// adjust nSourceIdx
				if (++nSourceIdx > arrSource.length - 1) {
					alert('this is the last photo. go to first.');
					nSourceIdx = 0;
				}
				playImage(nSourceIdx);
				break;
				
			case TVKEY.KEY_ENTER:
			case TVKEY.KEY_PANEL_ENTER:
			case TVKEY.KEY_PANEL_SOURCE:
				if (arrSource.length == 1) return;
				if (defPlayState == CSlideshow.Define.PLAY) {
					this.pause();
				}
				else {
					this.play();
				}
				break;
				
			case TVKEY.KEY_RW:
				if (arrSource.length == 1) return;
				iInterval.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_FF:
				if (arrSource.length == 1) return;
				iInterval.keyHandler(keyCode);
				break;
				
			case TVKEY.KEY_INFO:
				break;
				
			case TVKEY.KEY_PLAY:
				break;
				
			case TVKEY.KEY_PAUSE:
				break;
				
			case TVKEY.KEY_TOOLS:
				break;
				
			case TVKEY.KEY_BLUE:
				break;
				
			case TVKEY.KEY_MENU:
				location.reload();
				break;
		}
	}
	
	// public functions
	this.open = function () {
		alert(strName + ' open()');
		setImageViewerEnv();
		show();
		playImage(nSourceIdx);
		_THIS_.play();
	}
	this.close = function () {
		alert(strName + ' close()');
		hide();
	}
	this.play = function () {
		alert(strName + ' play()');
		if (defPlayState == CSlideshow.Define.PAUSE) {
			defPlayState = CSlideshow.Define.PLAY;
			
			if (!bPlayMode || bRendered) {
				iInterval.stop();
				iInterval.start();
				prepareNext();
			}
			// view 작업
		}
	}
	this.pause = function () {
		alert(strName + ' pause()');
		if (defPlayState == CSlideshow.Define.PLAY) {
			defPlayState = CSlideshow.Define.PAUSE;
			iInterval.stop();
			// view 작업
			
			if (!bPlayMode) {
				iImageViewer.stop();
//					bPrepared = true;
				bRendered = true;
			}
		}
	}
	this.addSource = function (arrImage) {
		if (arrSource == null) {
			arrSource = [];
		}
		arrSource = arrSource.concat(arrImage);
		
		if (arrSource.length > MAX_SOURCE) {
			var from = arrSource.length - MAX_SOURCE;
			arrSource = arrSource.slice(from);
			alert('Too many sources are added. Head of sources over '+MAX_SOURCE+' are deleted.');
		}
	}
	this.clearSource = function () {
		arrSource = null;
		nSourceIdx = 0;
	}
	this.addInterval = function (arrInterval) {
		iInterval.addInterval(arrInterval);
	}
	this.clearInterval = function () {
		iInterval.clearInterval();
	}
	this.setNotifier = function (count, fn) {
		fnNotifier = fn;
		nNotifyCount = count;
	}
	
	// private functions
	function show () {
		alert(strName + ' show()');
		iImageViewer.show();
		IVEvtReceiver.setMasterObject(iImageViewer);
		iView.show();
	}
	function hide () {
		alert(strName + ' iShow hide()');
		iImageViewer.hide();
		iView.hide();
	}
	function setImageViewerEnv () {
		iImageViewer.setFrameArea(0,0,960,540);
		iImageViewer.setOnNetworkError(function(){
			onNetworkError();
		});
		iImageViewer.setOnRenderError(function(){
			onRenderError();
		});
		iImageViewer.setOnBufferingComplete(function(){
			onBufferingComplete();
		});
		iImageViewer.setOnRenderingComplete(function(){
			onRenderingComplete();
		});
		iImageViewer.startSlideshow();
	}
	function playImage (nIdx) {
		alert(strName + ' playImage('+nIdx+')');
		var source = getSource(nIdx);
		if (source) {
			if (!iImageViewer.isValidType(source.type)) {
				alert(strName + ' not support format : ' + source.type);
				alert(source.filename);
				alert(source.url);
				onRenderError();
				return;
			}
			bPlayMode = true;
			bNetworkError = false;
			iImageViewer.play(source.url);
		}
	}
	function prepareNext() {
		alert(strName + ' prepareNext()');
		nNextIdx = nSourceIdx;
		if (++nNextIdx > arrSource.length - 1) {
			alert('this is the last photo. go to first.');
			nNextIdx = 0;
		}
		
		// 마지막에서 n개 전에 알린다.
		if (fnNotifier && (arrSource.length - nNextIdx - 1) == nNotifyCount) {
			fnNotifier();
		}
		
		var source = getSource(nNextIdx);
		if (source) {
			if (!iImageViewer.isValidType(source.type)) {
				alert(strName + ' not support format : ' + source.type);
				alert(source.filename);
				alert(source.url);
				bNextNotSupport = true;
				return;
			}
			bPlayMode = false;
			bPrepared = false;
			bRendered = false;
			
			// 이전 사진이 not support 였거나
			// network error 상황이었다면 NONE 으로 설정한다.
			var effect = defEffect;
			if (bNextNotSupport ||  bNetworkError) {
				effect = CImageViewer.Effect.NONE;
			}
			
			bNextNotSupport = false;
			bNetworkError = false;
			iImageViewer.prepareNext(source.url, effect);
		}
	}
	function nextSlide() {
		nSourceIdx = nNextIdx;
		if (bNextNotSupport) {
			onRenderError();
			return;
		}
		bEffectProcess = true;
		iImageViewer.showNow();
	}
	function onNetworkError () {
		alert(strName + ' onNetworkError()');
		bPrepared = true;
		bRendered = true;
		/**
		 * 여기서 Not Support 처리
		 * 사용자에게 알리고 끝낸다 or 사용자에게 알리고 계속한다.
		 */
		if (defPlayState == CSlideshow.Define.PLAY) {
			iInterval.stop();
			iInterval.start();
		}
	}
	function onRenderError () {
		alert(strName + ' onRenderError()');
		bPrepared = true;
		bRendered = true;
		/**
		 * 여기서 Not Support 처리
		 * 사용자에게 알리고 끝낸다 or 사용자에게 알리고 계속한다.
		 */
		if (defPlayState == CSlideshow.Define.PLAY) {
			iInterval.stop();
			iInterval.start();
		}
	}
	function onBufferingComplete () {
		alert(strName + ' onBufferingComplete()');
		bPrepared = true;
		if (!bPlayMode 
		&& defPlayState == CSlideshow.Define.PLAY
		&& iInterval.getState().timer != iInterval.START) {
			nextSlide();
		}
	}
	function onRenderingComplete () {
		alert(strName + ' onRenderingComplete()');
		bEffectProcess = false;
		bRendered = true;
		if (defPlayState == CSlideshow.Define.PLAY) {
			iInterval.stop();
			iInterval.start();
			prepareNext();
		}
	}
	function getSource(nIdx) {
		var retValue = null;
		if (arrSource && arrSource[nIdx]) {
			retValue = arrSource[nIdx];
		}
		alert(strName + ' getSource('+nIdx+') returns ' + retValue);
		return retValue;
	}
	
	function Interval () {
		var DEFAULT_INTERVAL = new CSlideshow.Interval('Default', 5000);	// default 5 sec.
		
		// timer state
		this.START = 1;		// Timer가 셋되서 돌고 있음.
		this.END = 2;		// Timer가 정해진 시간이 지나 event 발생시킨 후 끝났음.
		this.STOP = 3;		// Timer가 외부요인으로 cancel 되었음.
		
		var defTimerState = 0;
		var arrInterval = null;
		var nIntervalIdx = 0;
		
		var timer = null;
		var fnOnEvent = null;
		
		this.start = function () {
			alert('Interval timer start!');
			defTimerState = 1;
			timer = setTimeout(function(){
				alert('Interval timer End!');
				timer = null;
				defTimerState = 2;
				if (fnOnEvent) {
					fnOnEvent();
				}
			}, getInterval(nIntervalIdx).time);
		}
		this.stop = function () {
			if (timer) {
				alert('Interval timer stopped!');
				clearTimeout(timer);
				timer = null;
				defTimerState = 3;					
			}
			else {
				alert('Interval timer already ended.');
			}
		}
		this.setOnEvent = function (fn) {
			alert(fn);
			fnOnEvent = fn;
		}
		this.getState = function () {
			return {
				timer : defTimerState,
				Interval : getInterval(nIntervalIdx)
			}
		}
		this.addInterval = function (arr) {
			if (arrInterval == null) {
				arrInterval = [];
			}
			arrInterval = arrInterval.concat(arr);
			if (arrInterval.length > MAX_SOURCE) {
				var from = arrInterval.length - MAX_SOURCE;
				arrInterval = arrInterval.slice(from);
				alert('Too many sources are added. Head of sources over '+MAX_SOURCE+' are deleted.');
			}
		}
		this.clearInterval = function () {
			arrInterval = null;
			nIntervalIdx = 0;
		}
		function getInterval (nIdx) {
			var retValue = DEFAULT_INTERVAL;
			var constructor = arrInterval[nIdx].constructor;
			if (arrInterval && arrInterval[nIdx]
			&& constructor == CSlideshow.Interval) {
				retValue = arrInterval[nIdx];
			}
			alert(strName + ' getInterval('+nIdx+') returns ' + retValue.name + ',' + retValue.time);
			return retValue;
		}
		// for key handling
		this.keyHandler = function (keyCode) {
			keyCode = (keyCode === undefined) ? event.keyCode : keyCode;
			alert(strName + ' keyHandler('+keyCode+')');
			switch (keyCode) {
				case TVKEY.KEY_RW:
					this.stop();
					// adjust nIntervalIdx
					if (--nIntervalIdx < 0) {
						alert('this is the first Interval.');
						nIntervalIdx = 0;
					}
					this.start();
					break;
				case TVKEY.KEY_FF:
					this.stop();
					// adjust nIntervalIdx
					if (++nIntervalIdx > arrInterval.length - 1) {
						alert('this is the last Interval.');
						nIntervalIdx = arrInterval.length - 1;
					}
					this.start();
					break;
			}
		}
	}
	function View () {
		this.show = function () {
			
		}
		this.hide = function () {
			
		}
	}
}
